<?php
/**
*
* @package Vikingboard
* @author Tor Henning Ueland <tor.henning@vikingboard.com>
* @version 0.1b
*
* $Id: functions.php,v 1.38 2006/06/19 21:10:34 ueland Exp $
*
*/



/**
 * Set up driver types to use, don't touch if 
 * you don't know what you are doing
 * 
 * @todo Make a nice GUI to change this
 *
 */
class config_drivers
{
	var $dbdriver	=	"MySQL";
	var $maildriver	=	"php";
}
$drivers = new config_drivers;


//Set up error reporting handling
error_reporting(0);

//Only show error messages if the user wants to
if (@$_GET['error'] == 1)
{
	error_reporting(E_ALL);
}

//error reporting without the annoying notice-errors
if (@$_GET['notnotice'] == 1)
{
	error_reporting(E_ALL ^ E_NOTICE);
}

//Start the debug timer if we are debugging
if (@$_GET['debug']  == 1)
{
	class debug
	{
		var $debugdata;

		var $cachetime;
		var $sqltime;
	}
$debug	= new debug;
}



/**
 * Simple timer class, for timing and profiling
 *
 */
class Timer {

	var $starttime;
	var $endtime;
	
	function Timer() {
		$this->starttime = microtime();
		return true;
	}
	
	function stop_timer() {
		$this->endtime = microtime();
		return true;
	}
	
	function display_timer($decimals = 2) {
		$decimals = intval($decimals);
        if ($decimals > 8)
          $decimals = 8;
        if ($decimals < 0)
          $decimals = 0;
        $endtime = explode(" ", $this->endtime);
           $endtime = (float)$endtime[1] + (float)$endtime[0];
           $starttime = explode (" ", $this->starttime);
           $starttime = (float)$starttime[1] + (float)$starttime[0];
        return number_format($endtime - $starttime, $decimals); 
		return true;
	}
	
	/**
	 * Stops the timer, and displays the time taken
	 *
	 * @param int $decimals
	 * @return float $time
	 */
	function stop_and_display($decimals = 2) {
		$this->stop_timer();
		return $this->display_timer($decimals);
	}
}

//instantiating the class, and starting the timer
$timer = new timer;




//Let's parse all incoming data to make sure it's nice and clean

/**
 * Emulates MySQLs strip function, strips disturbing stuff
 * from strings going to SQL
 *
 * @param string $string
 * @return string $string
 */
function sql_escape_string($string) {
  
  	$string = str_replace("'", "\\'", $string);
  	$string = str_replace("\\n", "\\\\n", $string);
  	$string = str_replace("\\r", "\\\\r", $string);
  	
	return $string;
}

/**
 * Parse global arrays, and strips them of naughty stuff
 *
 * @param array $global
 * @param bool $magicquotes
 * @return array $global
 */
function parse_global($global, $magicquotes)
{
	foreach($global as $key => $value)
	{
		$key   = sql_escape_string($key);
		$value = sql_escape_string($value);
		if ($magicquotes == true)
			$value = strip_magic_quotes_gpc($value);
			
		$key   = trim($key);
		//$return = array();
		
		//let's recurse!
		if (is_array($value))
			$return[$key] = parse_global($value, $magicquotes);
		else
			$return[$key] = $value;
	}
	return $return;
}

//parse POST, GET and COOKIE, and make sure they are safe to use
$magicquotes = get_magic_quotes_gpc();
$_POST 	= parse_global($_POST, $magicquotes);
$_GET	= parse_global($_GET, $magicquotes);
$_COOKIE = parse_global($_COOKIE, $magicquotes);


//Include needed libraries
//the database file
if(defined('IN_ACP') == '1')
{
	require_once("inc/drivers/sql/adminsql_".$drivers->dbdriver.".php");
	$db	=	new sql();	
}
else
{
	require_once("inc/drivers/sql/sql_".$drivers->dbdriver.".php");
	$db	=	new sql();
}

//the configuration file
if (!(include_once("inc/conf.php")))
{
	die(print '<h2>forum errror</h2><b>could not get the config file.</b>');
}
$config_global		=	new config_global;


//database cache system
if (!(include_once("inc/lib/cache.php")))
{
	die(print '<h2>forum errror</h2><b>could not load the cache system.</b>');
}


//set cache and config
$cache	=	new cache();
$config =	new config;	//the site config class



//fetching the user file.
if (!(include_once("inc/user.php")))
{
	die(print '<h2>forum errror</h2><b>could not get the user file.</b>');
}
$user			=	new user();	//the user data class



//fetching the screen file.
if (!(include_once("inc/lib/screen.php")))
{
	die(print '<h2>forum errror</h2><b>could not get the output data file.</b>');
}
$screen = new screen;



//fetching the language file.
if(defined('IN_ACP') == '1') {
    if (!(include_once('inc/lang/'.$config->def_lang.'/admin.php')))
    {
    	die(print '<h2>forum errror</h2><b>could not get the language file.</b>');
    }
} else {

}
if (!(include_once('inc/lang/'.$config->def_lang.'/index.php')))
{
	die(print '<h2>forum errror</h2><b>could not get the language file.</b>');
}

//fetching the mail file.
if (!(include_once("inc/drivers/mail/".$drivers->maildriver.".php")))
{
	die(print '<h2>forum errror</h2><b>could not get the email driver file.</b>');
}



//Trigger error if the board is offline
if($config->active =='0' && !$user->g_view_offline || $config->forum_on =='0' && !$user->g_view_offline)
{
	error_msg("<b>turned off</b><br>the forum is currently turned off.", 1);
}


//Trigger error if the user is banned
if($user->g_viewforum !=1 || $user->group == 4) {
  	$lang_login	=	new lang_login;
	error_msg($lang_login->login_banned, 1);
}



/**
 * Returns the clock in unix timestamp, is to be improved
 * 
 * @param int $timestamp
 * @return int $time
 */
function forumtime($timestamp = false)
{
	global $config;
	
	//Either use timestamp or unix time
	if ($timestamp) 
		$time = $timestamp;
		
	elseif ($timestamp == false)
		$time = time();
	
	//Adjust it
	$adjust = $config->time_adjust;
	if(stristr($adjust, "-") === false) {
		return ($time + $adjust);
	} else {
		$adjust = str_replace("-", "", $adjust);
		return ($time - $adjust);
	}
}



/**
 * Output a error
 *
 * @param string $msg
 * @param bool $add_wrapper
 */
function error_msg($msg, $add_wrapper)
{
	//uck, globalise
	global $cache, $config, $screen, $user;
	
	//If something is really fucked up, we are only going to show the error
	if(!is_object($cache) || !is_object($screen) || !$user) {
		die("<b>Vikingboard fatal error</b><br />\n{$msg}");
 	}
	
	//Set language
	if (!(include_once('inc/lang/'.$config->def_lang.'/index.php')))
	{
		include_once('inc/lang/1/index.php');
	}
	$lang_global    =    new lang_global;
	
	if ($user->skin_set)
	{
		$skin_set = $user->skin_set;
	}
	else
	{
		$skin_set = $config->def_tpl;
	}
	//add template
	$tohtml = array("message" => $msg,
					"imgset" => $skin_set);
	$screen->add_html('message', $tohtml);

	//wrap?
	if($add_wrapper)
	{
		$navigation["index.php"]	=	$lang_global->info;
		$screen->add_wrapper($lang_global->info, $navigation);
	}
	//lets print it
	$screen->print_page();

	//and stop the script
	exit();
}



/**
 * Output a notification with a link. 
 *
 * @param string $msg
 * @param string $target
 * @param string $link_text
 */
function notify($msg, $target, $link_text)
{
	//uck, globalise
	global $config, $screen;
		
	//Set language
	$lang_global    =    new lang_global;
	
	if ($user->skin_set)
	{
		$skin_set = $user->skin_set;
	}
	else
	{
		$skin_set = $config->def_tpl;
	}
	
	//add template
	$tohtml = array("message" => $msg,
					"imgset" => $skin_set,
					'target' => $target,
					'link' => $link_text);
					
	$screen->add_html('notification', $tohtml);

	//wrap it
	$navigation["index.php"]	=	$lang_global->notify;
	$screen->add_wrapper($lang_global->notify, $navigation);
	
	//lets print it
	$screen->print_page();

	//and stop the script
	exit();
}




/**
 * Verifies that a string actually is an e-mail
 *
 * @param string $email
 * @return true/false
 */
function is_valid_email_address($email){

	$qtext = '[^\\x0d\\x22\\x5c\\x80-\\xff]';

	$dtext = '[^\\x0d\\x5b-\\x5d\\x80-\\xff]';

	$atom = '[^\\x00-\\x20\\x22\\x28\\x29\\x2c\\x2e\\x3a-\\x3c'.
			'\\x3e\\x40\\x5b-\\x5d\\x7f-\\xff]+';

	$quoted_pair = '\\x5c[\\x00-\\x7f]';

	$domain_literal = "\\x5b($dtext|$quoted_pair)*\\x5d";

	$quoted_string = "\\x22($qtext|$quoted_pair)*\\x22";

	$domain_ref = $atom;

	$sub_domain = "($domain_ref|$domain_literal)";

	$word = "($atom|$quoted_string)";

	$domain = "$sub_domain(\\x2e$sub_domain)*";

	$local_part = "$word(\\x2e$word)*";

	$addr_spec = "$local_part\\x40$domain";

	return preg_match("!^$addr_spec$!", $email) ? 1 : 0;
}

function strip_magic_quotes_gpc ($input)
{
	if (is_array($input) === true)
	{
		foreach ($input as $k => $v)
		{
			if (is_array($v) === true)
			{
				$input[$k] = strip_magic_qoutes_gpc($v);
			}
			else 
			{
				$input[$k] = stripslashes($v);
			}
		}
	}

	else 
	{
		$input = stripslashes($input);
	}
	return $input;
}
?>