<?php
/**
*
* @name post_parser
*
* @desc	This file is the post parser, this contains a function that takes post data and makes
*		it look good and make sure that it`s coded good and nicely. Includes a nice BBCode parser.
* @todo We need custom BBCode and a more adaptable BBCode-engine. Regex is *not* optimal.
* @author Jørgen Tellnes
* @version 0.1b
*
* @example  include post_parser.php;
* 			$post_parser = new post_parser;
* 			$parsedpost = $post_parser->post_parse($_POST['post']);
*
* $Id: post_parser.php,v 1.13 2006/06/16 18:40:57 ueland Exp $
*/

class post_parser {
	
	
	function post_parse($post, $parse_smileys)
	{
		global $db;
		//starting with cleaning out bad stuff from the text and makes it look good
		
		//removing bad/naughty/other stuff
		$post	=	str_replace("<", "&lt;", $post);
		$post	=	str_replace(">", "&gt;", $post);
		$post	=	nl2br($post);
				
		//bad words
		$db_word	=	$db->get_badwords();
		
		while($db_words = $db_word->row())
		{
			$wfrom	.=	$db_words->name;
			$wto	.=	$db_words->nname;
			$post	=	str_replace("$wfrom","$wto", $post);
		}
		
		//smilies
		if($parse_smileys == 1) {
			$db_smiley	=	$db->get_smileys();
			while($db_smile = $db_smiley->row())
			{
				$post	=	str_replace(''.$db_smile->code.'','<!--smile_'.$db_smile->code.'--><img src="'.$db_smile->url.'" title="'.$db_smile->name.'" alt="' . $db_smile->name . '" /><!--smilestop-->', $post);
			}
		}
				
		//Starting to parse [bb]code[/bb]
			
			//another BBcode for URL :p
			$post   =   preg_replace("#\[url=http://(.+?)\](.+?)\[/url\]#is", "<a href=\"http://\\1\">\\2</a>", $post);
			
			//BBcode for URL 
			$post   =   preg_replace("#\[url=(.+?)\](.+?)\[/url\]#is", "<a href=\"http://\\1\">\\2</a>", $post);
			
			//lazy linking with http://
			$post   =   preg_replace("#\[url\]http://(.+?)\[/url\]#is", "<a href=\"http://\\1\">\\1</a>", $post);
			
			//lazy linking
			$post   =   preg_replace("#\[url\](.+?)\[/url\]#is", "<a href=\"http://\\1\">\\1</a>", $post);
			
			//Auto-change of urls to different stuff
			// $post   =   preg_replace("#www\.(.+?)#is", "<a href=\"http://www.\\1\">www.\\1</a>", $post); // doesnt work. Fix later.
		
			//BBcode for EMAILS
			$post   =   preg_replace("#\[email=(.+?)\](.+?)\[/email\]#is", "<a href=\"mailto:\\1\">\\2</a>", $post);
			
			//BBcode for lazy EMAILS
			$post   =   preg_replace("#\[email\](.+?)\[/email\]#is", "<a href=\"mailto:\\1\">\\1</a>", $post);
			
			//BBcode for QUOTES
			
				//if there is no =-character, do it the lazy way:
				$post = preg_replace("#\[quote\]#is", "<fieldset class=\"quote\"><legend>QUOTE:</legend><p class=\"nopadding\">", $post);
				
				//else, do it properly
				$post = preg_replace("#\[quote=(.+?)\]#is", "<fieldset class=\"quote\"><legend>QUOTE: $1</legend><p class=\"nopadding\">", $post);
				
				
				//then the data and the footer
				$post = preg_replace("#(.+?)\[/quote\]#is", "" . str_replace("<br>", "", "$1") . "</p></fieldset>", $post);

			//BBcode for HTML
			$post   =   preg_replace("#\[html\](.+?)\[/html\]#is", $this->highlight_html("\$1"), $post);

			//Bbcode for PHP
			$post   =   preg_replace("#\[php\](.+?)\[/php\]#is", "<div class=\"phpbox\">" . $this->highlight_php("\\1") . "</div>", $post); //broken
			
			//BBcode for SQL
			$post   =   preg_replace("#\[sql\](.+?)\[/sql\]#is", "<div class=\"sqlbox\"><p class=\"sqlboxheader\">SQL</p>\\1</div>", $post);
			
			//BBcode for "code"
			$post   =   preg_replace("#\[code\](.+)\[/code\]#is", "<div class=\"code\"><p class=\"codeheader\">CODE</p><pre><code>$1</code></pre></div>", $post);
			
			//BBCode for images
			$post   =   preg_replace("#\[img\](.+?)\[/img\]#is", "<img src=\"\\1\" alt=\"\" />", $post);
			
			//BBCode for images w/explaination below (bildetekst)
			$post   =   preg_replace("#\[img=(.+)\](.+?)\[/img\]#is", "<div class=\"imagebox\"><img src=\"\\1\" alt=\"\" /><p class=\"imagetext\">\\2</p></div>", $post);
			
			//BBCode for deleted text
			$post	=	preg_replace("#\[del\](.+?)\[/del\]#is", "<del>$1</del>", $post);
			
			//BBcode for headings
			$post	=	preg_replace("#\[h([1-6])\](.+?)\[/h\]#is", "<h$1 class=\"nopadding\">$2</h$1>", $post);
			
			//BBcode for <abbr title="#">
			$post	=	preg_replace("#\[abbr=(.+?)\](.+?)\[/abbr\]#is", "<abbr title=\"$1\">$2</abbr>", $post);
			
			//BBCode for <acronym title="#">
			$post	=	preg_replace("#\[acronym=(.+?)\](.+?)\[/acronym\]#is", "<acronym title=\"$1\">$2</acronym>", $post);
			
			//BBcodes for bold text			
			$post	=	preg_replace("#\[b\](.+?)\[/b\]#is", "<strong>$1</strong>", $post); 
						
			//BBcodes for underline text
			$post	=	preg_replace("#\[u\](.+?)\[/u\]#is", "<span class=\"underline\">\\1</span>", $post);
						
			//BBcode for italic text
			$post	=	preg_replace("#\[i\](.+?)\[/i\]#is", "<em>$1</em>", $post);
			
			//BBcode for coloured text
			$post	=	preg_replace("#\[color=(.+?)\](.+?)\[/color\]#is", "<span class=\"color-\\1\">\\2</span>", $post);
			
			//BBcode for other fonts text
			$post	=	preg_replace("#\[font=(.+?)\](.+?)\[/font\]#is", "<span style=\"font-family:\\1;\">\\2</span>", $post);

			//BBcode for size
			$post	=	preg_replace("#\[size=(.+?)\](.+?)\[/size\]#is", "<span style=\"font-size:\\1px;\">\\2</span>", $post);
					
			//BBcode for listings
			
			// a bit trixy, indeed.
			
			//bit more cleanup
			// $post	  =	  str_replace("/<br>|<br \/>/| <\br>", "\n", $post); // ?
			
			//stripping messes up the urls, not good
			//$post	=	stripslashes($post);
				
		
		return $post;
		
	}

	function post_deparse($post, $nobr = FALSE) {

		//================================
		//deparses some important things to make the post box look better
		//================================
		$post	=	str_replace("&lt;", "<", $post);
		$post	=	str_replace("&gt;", ">", $post);
		if ($nobr != TRUE)
		{
			$post	=	str_replace("<br>", "\n", $post);
			$post	=	str_replace("<br />", "\n", $post);
		}
		else 
		{
			$post	=	str_replace("<br>", "", $post);
			$post	=	str_replace("<br />", "", $post);
		}
		//================================
		// Removing editby-stuff
		//================================
		$post 	=	preg_replace("#<!--startedit-->(.+?)<!--endedit-->#is", '', $post);
		
		//================================
		//	Parse smilies into tags
		//================================
		$post	=	preg_replace("#<!--smile_(.+?)-->(.+?)<!--smilestop-->#is", "\\1", $post);

		//================================
		// Deparsing BBCodes
		//================================
	
		//starting with the quoting, headers first!
		$post = preg_replace("#<fieldset class=\"quote\"><legend>QUOTE: (.+?)</legend><p class=\"nopadding\">#is", "[quote=$1]", $post);
	
		//then the footer
		$post = preg_replace("#(.+?)</p></fieldset>#is", "$1[/quote]", $post);
	
		//deparsing [url], both the lazy and the standard will be parsed the same way
		$post   =   preg_replace("#<a href=\"(.+?)\">(.+?)</a>#is", "[url=$1]$2[/url]", $post);
	
		//deparsing [img] with image description
		$post	=	preg_replace("#<div class=\"imagebox\"><img src=\"(.+?)\" alt=\"\" /><p class=\"imagetext\">(.+?)</p></div>#is", "[img=$1]$2[/img]", $post);
	
		//deparsing [img]
		$post   =   preg_replace("#<img src=\"(.+?)\" alt=\"\" />#is", "[img]$1[/img]", $post);
	
	
		//deparsing [a=]
		$post	=	preg_replace("#<abbr title=\"(.+?)\">(.+?)</abbr>#is", "[a=$1]$2[/a]", $post);
	
		//deparsing [del]
		$post	=	preg_replace("#<del>(.+?)</del>#is", "[del]$1[/del]", $post);
	
		//deparsing headings
		$post	=	preg_replace("#<h([1-6]) class=\"nopadding\">(.+?)</h([1-6])>#is", "[h$1]$2[/h]", $post);
	
		//deparsing [php]
	
		//deparsing [code]
		$post   =   preg_replace("#<div class=\"code\"><code>(.+?)</code></div>#is", "[code]$1[/code]", $post);
	
		//deparsing [b]
		$post   =   preg_replace("#<strong>(.+?)</strong>#is", "[b]$1[/b]", $post);
	
		//deparsing [i]
		$post   =   preg_replace("#<em>(.+?)</em>#is", "[i]$1[/i]", $post);
	
		//deparsing [u]
		$post   =   preg_replace("#<span class=\"underline\">(.+?)</span>#is", "[u]$1[/u]", $post);
	
		//deparsing [color]
		$post   =   preg_replace("#<span class=\"color(.+?)\">(.+?)</span>#is", "[color=$1]$2[/color]", $post);
	
		//deparsing [font]
		$post	=	preg_replace("#<span style=\"font-family:(.+?);\">(.+?)</span>#is", "[font=$1]$2[/font]", $post);
	
		//deparsing [size]
		$post	=	preg_replace("#<span style=\"font-size:(.+?)px;\">(.+?)</span>#is", "[size=$1]$2[/size]", $post);
	
		$post = stripslashes($post);
	
		return $post;
	}

	function highlight_html($code) { //HTML highlighting. Standard colors, or editable via Admin CP? Going standard at the moment
	
		/**
		*
		* @todo add more tags, ex. all form tags, including textfields
		*
		*/
		
		//coloring is a steady horse
		
		// highlighting of generic tags
		$code = preg_replace("#&lt;(.+?)&gt;#is", "<span style=\"color:#000099;\">&lt;\\1&gt;</span>", $code); 
		
		// <a>
		$code = preg_replace("#&lt;a(.+?)&gt;(.+?)&lt;/a&gt;#is", "<span style=\"color:#006600;\">&lt;a\\1&gt;</span>\\2<span style=\"color:#006600;\">&lt;/a&gt;</span>\n\r", $code); 
		
		// <img>
		$code = preg_replace("#&lt;img(.+?)&gt;#is", "<span style=\"color:#990099;\">&lt;img\\1&gt;</span>", $code);
		
		// <input>
		$code = preg_replace("#&lt;input(.+?)&gt;#is", "<span style=\"color:#FF9900;\">&lt;input\\1&gt;</span>", $code);
		
		// <style>
		$code = preg_replace("#&lt;style&gt;(.+?)&lt;/style&gt;#is", "<span style=\"color:#990099;\">&lt;style&gt;</span>\\1<span style=\"color:#990099\">&lt;/style&gt;</span>", $code);
		
		// <!-- and -->
		$code = preg_replace("#&lt;!--(.+?)--&gt;#is", "<span style=\"color:#999999;\">&lt;!--\\1--&gt;</span>", $code);
		
		// <script> with closing tag
		$code = preg_replace("#&lt;script(.+?)&gt;(.+?)&lt;/script&gt;#is", "<span style=\"color:#990000;\">&lt;script\\1&gt;</span>\\2<span style=\"color:#990000;\">&lt;/script&gt</span>", $code);
		
		// <script> w/o closing tag
		$code = preg_replace("#&lt;script(.+?)&gt;#is", "<span style=\"color:#990000;\">&lt;script\\1&gt;</span>", $code);
		
		// <form>
		$code = preg_replace("#&lt;form(.+?)&gt;#is", "<span style=\"color:#FF9900;\">&lt;form\\1&gt;</span>", $code);
		
		// </form>
		$code = preg_replace("#&lt;/form(.+?)&gt;#is", "<span style=\"color:#FF9900;\">&lt;/form\\1&gt;</span>", $code);
		
		// generic tag attributes
        function attr($match) {
            return htmlspecialchars("<" . $match[1] . " ")
            . preg_replace("#([a-z\-]+)=(&quot;.*?&quot;)($| |\n)#", "\\1=<span style=\"color:#0000FF;\">\\2</span>\\3", $match[2])
            . htmlspecialchars(">");
        }
        
        $code = preg_replace_callback("#&lt;([a-z0-9]+) (([a-z\-]+=&quot;(.*?)&quot; *)*.*?)&gt;#is", 'attr', $code); 
       
		//indeed.
		return $code;
	}

	function highlight_php($code) { // highlights php-code
		// $code   = stripslashes($code); //if it starts to mess, remove the comment.
		$code = highlight_string($code, true);
		return $code;
	}
}


?>